---------------------------------------------------------------------------
-- Transformer v 1.2 - Talent/Gear switcher by Acuzod - acuzod@gmail.com --
---------------------------------------------------------------------------

--------------------
-- SavedVariables --
--------------------
if Transformer_Settings == nil then
  Transformer_Settings = {
	MinimapPos = 45,
    Transformer_Primary_Name = nil,
    Transformer_Primary_Equipment = nil,
    Transformer_Seconday_Name = nil,
    Transformer_Secondary_Equipment = nil
  }
end

---------------------
-- Local Variables --
---------------------

-- Filters Based on DualSpeccer by Jeffrey0
local filter1 = string.gsub(ERR_SPELL_UNLEARNED_S:gsub('%.', '%.'), '%%s', '(.*)')
local filter2 = string.gsub(ERR_LEARN_ABILITY_S:gsub('%.', '%.'), '%%s', '(.*)')
local filter3 = string.gsub(ERR_LEARN_SPELL_S:gsub('%.', '%.'), '%%s', '(.*)')
local isTransforming = false -- Used to prevent duplicate messages
local Transformer_Active_Set = nil
local Transformer_Active_Name = nil
local Transformer_Active_IconTexture = nil
local Transformer_Inactive_Name = nil


-------------------
-- Event Watcher --
-------------------

function Transformer_OnEvent(self,event,eventArg1,...) 
  
  
    if (event == "ADDON_LOADED" and eventArg1 == "Transformer") then
        -- Transformer_MinimapButton_Icon:SetTexture(Transformer_Active_IconTexture);
        Transformer_MinimapButton_Reposition();
    end
          
    if (event == "UNIT_SPELLCAST_SUCCEEDED") and eventArg1 == "player" then
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_START")
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
        if GetActiveTalentGroup(0) == 1 then
            if (Transformer_Settings.Transformer_Primary_Equipment ~= nil) and (Transformer_Settings.Transformer_Primary_Equipment ~= "None") then
              UseEquipmentSet(Transformer_Settings.Transformer_Primary_Equipment) 
            end 
          else
            if (Transformer_Settings.Transformer_Secondary_Equipment ~= nil) and (Transformer_Settings.Transformer_Secondary_Equipment ~= "None") then
              UseEquipmentSet(Transformer_Settings.Transformer_Secondary_Equipment) 
            end
          end      
    end
    
    if ((event == "UNIT_SPELLCAST_INTERRUPTED" or event == "UNIT_SPELLCAST_FAILED")  and eventArg1 == "player") then
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_FAILED")
        transformerFrame:UnregisterEvent("UNIT_SPELLCAST_START")
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", filter)
        print("Transformer: Transformation interrupted")

    end
    
    if (event == "PLAYER_TALENT_UPDATE") then
     Transformer_LoadTalentsInfo()
    end
             
end

------------------------------------------
-- Minimap Button Position and Movement --
------------------------------------------

function Transformer_MinimapButton_Reposition()
	Transformer_MinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(Transformer_Settings.MinimapPos)),(80*sin(Transformer_Settings.MinimapPos))-52)
end

-- Only while the button is dragged this is called every frame
function Transformer_MinimapButton_DraggingFrame_OnUpdate()

	local xpos,ypos = GetCursorPosition()
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin-xpos/UIParent:GetScale()+70 -- get coordinates as differences from the center of the minimap
	ypos = ypos/UIParent:GetScale()-ymin-70

	Transformer_Settings.MinimapPos = math.deg(math.atan2(ypos,xpos)) -- save the degrees we are relative to the minimap center
	Transformer_MinimapButton_Reposition() -- move the button
end

-------------------------------
-- Minimap Button Click --
-------------------------------'

-- Put your code that you want on a minimap button click here.  arg1="LeftButton", "RightButton", etc
function Transformer_MinimapButton_OnClick(button)

  button = GetMouseButtonClicked();
  
  if (button == "LeftButton") then
        isTransforming = true     
        if GetNumTalentGroups(0) == 2 then
          PlaySoundFile("Interface\\AddOns\\Transformer\\transformer.mp3")
            if GetActiveTalentGroup(0) == 1 then
                SetActiveTalentGroup(2)
            else
                SetActiveTalentGroup(1)
            end     
        end
    end
    
    if (button == "RightButton") then
     InterfaceOptionsFrame_OpenToCategory("Transformer")
    end
    
   
end

-----------------------
-- Load Talents Info --
-----------------------

function Transformer_LoadTalentsInfo()
  local numPoints
  local Primary_Name
  local Primary_IconTexture
  local Secondary_Name
  local Secondary_IconTexture
  local testvalue
    
  -- Get primary talent spec info
  numPoints = 0
  for i = 1, GetNumTalentTabs() do
    id, name, description, iconTexture, pointsSpent, background, previewPointsSpent = GetTalentTabInfo(i,0,0,1)
    if pointsSpent > numPoints then
      Primary_Name = name
	  Primary_IconTexture = iconTexture
	  numPoints = pointsSpent
    end 
  end 
  
    
  -- If user has set primary spec name and/or equipment set in Transformer, use those instead
  if Transformer_Settings.Transformer_Primary_Name ~= nil then
    Primary_Name = Transformer_Settings.Transformer_Primary_Name
  end
  
  if (Transformer_Settings.Transformer_Primary_Equipment ~= nil) and (Transformer_Settings.Transformer_Primary_Equipment ~= "None")then
     icon, setID = GetEquipmentSetInfoByName(Transformer_Settings.Transformer_Primary_Equipment);
    if icon ~= nil then
      Primary_IconTexture = "Interface\\Icons\\" .. icon
    end
  end
        
  -- Get secondary talent spec info if exists  
  numPoints = 0
  if GetNumTalentGroups(0) == 2 then 
    for i = 1, GetNumTalentTabs() do
      id, name, description, iconTexture, pointsSpent, background, previewPointsSpent = GetTalentTabInfo(i,0,0,2)
      if pointsSpent > numPoints then
        Secondary_Name = name
	    Secondary_IconTexture = iconTexture
	    numPoints = pointsSpent
      end 
    end   
        
    -- If user has set secondary spec name and/or equipment set, use those instead
    if Transformer_Settings.Transformer_Secondary_Name ~= nil then
      Secondary_Name = Transformer_Settings.Transformer_Secondary_Name
    end
  
    if (Transformer_Settings.Transformer_Secondary_Equipment ~= nil) and (Transformer_Settings.Transformer_Secondary_Equipment ~= "None") then
       icon, setID = GetEquipmentSetInfoByName(Transformer_Settings.Transformer_Secondary_Equipment);
      if icon ~= nil then
        Secondary_IconTexture = "Interface\\Icons\\" .. icon
      end
    end
  end  
    
  -- Set Active/Inactive info    
  if GetActiveTalentGroup(0) == 1 then
    Transformer_Active_Name = Primary_Name
    Transformer_Active_Set = Transformer_Settings.Transformer_Primary_Equipment
    Transformer_Active_IconTexture = Primary_IconTexture
    Transformer_Inactive_Name = Secondary_Name
  else
    Transformer_Active_Name = Secondary_Name
    Transformer_Active_Set = Transformer_Settings.Transformer_Secondary_Equipment
    Transformer_Active_IconTexture = Secondary_IconTexture
    Transformer_Inactive_Name = Primary_Name
  end
        
  -- Set minimap icon 
  Transformer_MinimapButton_Icon:SetTexture(Transformer_Active_IconTexture)
  
  if (isTransforming == true) then  
    print("Transformer: You are " .. Transformer_Active_Name)
    isTransforming = false;
  end
    
end -- Transformer_LoadTalentsInfo()


--------------------------------
-- Load Equipment Dropdowns --
--------------------------------
function TransformerEquipmentDropDown_Initialize()
  local info = UIDropDownMenu_CreateInfo();
  local count = GetNumEquipmentSets();
  info.text = "None";
  info.func = TransformerEquipmentDropDownButton_OnClick;
  info.notCheckable = true;
  UIDropDownMenu_AddButton(info);
  for i=1, count, 1 do
    name, icon, lessIndex = GetEquipmentSetInfo(i);
    info.text = name;
    info.func = TransformerEquipmentDropDownButton_OnClick;
	info.checked = nil;
	info.icon = icon
	UIDropDownMenu_AddButton(info);
  end
end
         
function TransformerEquipmentDropDown_OnLoad(self)
  UIDropDownMenu_Initialize(self, TransformerEquipmentDropDown_Initialize);
  UIDropDownMenu_SetButtonWidth(self, 24);
  UIDropDownMenu_JustifyText(self, "LEFT")
end

function TransformerEquipmentDropDownButton_OnClick(self)
  UIDropDownMenu_SetText(UIDROPDOWNMENU_OPEN_MENU, self:GetText());
end


--------------------------------
-- Set Minimap Button Tooltip --
--------------------------------
function Transformer_MinimapButton_OnEnter(self)
  if (self.dragging) then
    return
  end
	
  GameTooltip:SetOwner(self, "ANCHOR_LEFT")
  GameTooltip:SetText("Transformer")
	
  if GetNumTalentGroups(0) == 1 then
    GameTooltip:AddLine("Current spec: " .. Transformer_Active_Name, 1, 1, 1)
	GameTooltip:AddLine("No secondary spec available",1)
  else
    GameTooltip:AddLine("Current spec: " .. Transformer_Active_Name, 1, 1, 1)
	GameTooltip:AddLine("Click to transform to: " .. Transformer_Inactive_Name, 1, 1, 1)
  end   
	
  GameTooltip:Show()
end

------------------------
-- Talent Change Hook --
------------------------
local orgSetActiveTalentGroup = _G.SetActiveTalentGroup
function SetActiveTalentGroup(...)
  if (isTransforming == true) then 
    print("Transformer: Transforming to " .. Transformer_Inactive_Name .. "...")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
    transformerFrame:RegisterEvent("UNIT_SPELLCAST_START")	
    ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", filter)
  end
  return orgSetActiveTalentGroup(...)	
end

function filter(self, event, msg)
  if strfind(msg, filter1) or strfind(msg, filter2) or strfind(msg, filter3) then
    return true
  end
end
	
	

------------------------
-- Options Panel Code --
------------------------	
function TransformerOptionsPanel_OnLoad(panel)
   panel.name = "Transformer"
   panel.okay = function (self) TransformerOptionsPanel_Close(); end;     
   panel.cancel = function (self)  TransformerOptionsPanel_CancelorLoad();  end;
   panel.refresh = function (self)  TransformerOptionsPanel_CancelorLoad();  end;
   InterfaceOptions_AddCategory(panel);  
end
    
function TransformerOptionsPanel_Close()
   Transformer_Settings.Transformer_Primary_Name = TransformerPrimaryNameTextbox:GetText();  
   Transformer_Settings.Transformer_Secondary_Name = TransformerSecondaryNameTextbox:GetText(); 
   Transformer_Settings.Transformer_Primary_Equipment = UIDropDownMenu_GetText(TransformerPrimaryEquipmentDropDown);
   Transformer_Settings.Transformer_Secondary_Equipment = UIDropDownMenu_GetText(TransformerSecondaryEquipmentDropDown);
   Transformer_LoadTalentsInfo()
end
    
function TransformerOptionsPanel_CancelorLoad()
  if Transformer_Settings.Transformer_Primary_Name ~= nil then
    TransformerPrimaryNameTextbox:SetText(Transformer_Settings.Transformer_Primary_Name);
  end
  
  if Transformer_Settings.Transformer_Secondary_Name ~= nil then
    TransformerSecondaryNameTextbox:SetText(Transformer_Settings.Transformer_Secondary_Name);
  end
 
 if Transformer_Settings.Transformer_Primary_Equipment ~= nil then
   UIDropDownMenu_SetText(TransformerPrimaryEquipmentDropDown, Transformer_Settings.Transformer_Primary_Equipment);
 end
 
 if Transformer_Settings.Transformer_Secondary_Equipment ~= nil then
  UIDropDownMenu_SetText(TransformerSecondaryEquipmentDropDown, Transformer_Settings.Transformer_Secondary_Equipment);
 end
end
    
  
    
      
